==> irc-transport-0.1b-1.40.linux.tar.gz: irc-transport-0.1b-1.40/src/util.c <==
/*
IRC-Transport util.c: general utility functions for IRC-transport 
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include "irctrans.h"

/*
sets a mode for the user (ops and voice)
*/
void irc_set_mode(user u,int ops,int voice)
/*
returns;
 nothing
input:
 users u: the user to update
 int ops: the new ops vaule
 int voice: the new voice value
*/
{
    if(u==NULL) return;
    u->ops=ops;
    u->voice=voice;
}

/*
compares two nicknames, ignoreing ops and voice flags.
*/
int irc_nick_cmp(char *nick1,char *nick2)
/*
returns:
 -1 if either are NULL
 return value from strcasecmp for the two strings
input:
 char *nick1: duh
 char *nick2: duh
*/
{
    char *n1=nick1,*n2=nick2;
    if(n1==NULL||n2==NULL) return -1;
    if(n1[0]=='@'||n1[0]=='+') n1++;
    if(n2[0]=='@'||n2[0]=='+') n2++;
    return strcasecmp(n1,n2);
}

/*
strips out those nasty color codes
overwrites input with the new data
*/
void irc_strip_mirc_codes(char *input)
/*
returns:
 nothing
input:
 char *input: the string to convert, this gets overwritten iwth the result *MUST NOT BE READ ONLY* 
*/
{
    char *curr=input,*write=input;
    int a,w;
    long c;
    if(input==NULL) return;
    for(a=0,c=0,w=0;a<strlen(input);a++)
    {
        c=(long)curr[a];
        /*(  bold||underline||reverse||  plain)*/
        if(c==0x2||c==0x1F  ||c==0x16||c==0x0F)
        {
            /* skip this char */
            continue;
        }
        else if(c==0x3) /* one of these codes: # ## #,# #,## ##,# ##,## */
        { 
            c=curr[++a];
            if(isdigit(c))
            { /* # ## #,# #,## ##,# or ##,## */
                c=curr[++a];
                if(isdigit(c))
                { /* ## ##,# ##,## */
                    if(c==',')
                    { /* ##,# ##,## */
                        c=curr[a+2];
                        if(isdigit(c)) c=curr[++a];
                    }
                }
                else if(c==',')
                { /* #,# or #,## */    
                    c=curr[a+2];
                    if(isdigit(c)) c=curr[++a];
                }
            }
        }
        /* write the next char */
        write[w++]=(char)c;
    }
    write[w]='\0';
}

/* 
function to match '#Jabber' with 'jabber'
or '.jabber' with '&jabber'
NULL != NULL
*/
int irc_channel_cmp(const char *chan1,const char *chan2)
/*
returns:
 -1 if either are NULL
 the return value from strcasecmp
input:
 const char *char1,const char *char2: the values to compare
*/
{
    pool p;
    int ret=-1;
    char *c1,*c2;
    if(chan1==NULL||chan2==NULL) return ret;
    p=pool_new();
    c1=pstrdup(p,chan1);
    c2=pstrdup(p,chan2);
    /* normalize all channel names
        #foo => foo
        foo  => foo
        &foo => .foo
        .foo => .foo
    */
    if(c1[0]=='#') c1++;
    if(c1[0]=='&') c1[0]='.';
    if(c2[0]=='#') c2++;
    if(c2[0]=='&') c2[0]='.';
    if(c1!=NULL&&c2!=NULL)
        ret=strcasecmp(c1,c2);
    pool_free(p);
    return ret;
}

/*
extracts the channel name from a jid string
*/
char *irc_get_channel(pool p, char *input)
/*
returns:
 a pool'd copy o fthe channel using the supplied pool
 NULL if it could not find the channel
input:
 pool p: a pool to use to copy the data
 char *input: the jid string to extract from
*/
{
    char *curr,*next;
    char *ret=pstrdup(p,input);
    spool s;
    if(p==NULL||input==NULL) return NULL;
    curr=ret;
    next=strchr(curr,'%');
    if(next==NULL) next=strchr(curr,'@');
    if(next==NULL) return NULL;
    next[0]='\0';
    s=spool_new(p);
    if(curr[0]=='.') curr[0]='&';
    if(curr[0]!='#'&&curr[0]!='&') 
        spooler(s,"#",curr,s);
    else
        spooler(s,curr,s);
    return spool_print(s);
}

/*
extracts the server name from a jid string
*/
char *irc_get_server(pool p,char *input)
/*
returns:
 a pool'd copy of the server name using the supplied pool
 NULL if it could not find the server name
input:
 pool p: a pool to use to copy the data
 char *input: the jid string to extract from
*/
{
    char *curr,*next;
    char *ret=pstrdup(p,input);
    if(p==NULL||input==NULL) return NULL;
    curr=ret;
    next=strchr(curr,'%');
    if(next==NULL) 
        return pstrdup(p,xmlnode_get_tag_data(ircdata->cfg, "default/server"));
    curr=next+1;
    ret=curr;
    next=strchr(curr,'%');
    if(next==NULL) next=strchr(curr,'@');
    if(next==NULL) 
        return pstrdup(p,xmlnode_get_tag_data(ircdata->cfg, "default/server"));
    next[0]='\0';
    return ret;
}

/*
extracts the port name from a jid string
*/
char *irc_get_port(pool p,char *input)
/*
returns:
 a pool'd copy of the port string using the supplied pool
 NULL if it could not find the port
input:
 pool p: a pool to use to copy the data
 char *input: the jid string to extract from
*/
{
    char *curr,*next;
    char *ret=pstrdup(p,input);
    if(p==NULL||input==NULL) return NULL;
    curr=ret;
    next=strchr(curr,'%');
    if(next==NULL) 
        return pstrdup(p,xmlnode_get_tag_data(ircdata->cfg, "default/port"));    
    next=strchr(curr,'%');
    if(next==NULL)
        return pstrdup(p,xmlnode_get_tag_data(ircdata->cfg, "default/port"));    
    curr=next+1;
    next=strchr(curr,'@');
    if(next==NULL);
        return pstrdup(p,xmlnode_get_tag_data(ircdata->cfg, "default/port"));    
    ret=curr;
    next[0]='\0';
    return ret;
}

/* send an outgoing line to the connection */
void svc_irc_line(irc_session s, char *cmd, char *data, int argc, ...)
{
    static char buff[512], args[512];
    char *str;
    int strsz, i, argsz, flag;
    va_list ap;

    va_start(ap, argc);
    argsz = 0;
    args[argsz] = '\0';
    for(i = 0; i < argc; i++)
    {
        str = va_arg(ap,char *);
        if(str == NULL) continue;
        if(strlen(str) + argsz >= 500) continue;
        strcat(args,str);
        argsz += strlen(str);
        strcat(args," ");
        argsz++;
    }
    va_end(ap);

    if(data == NULL)
    {
        strsz = snprintf(buff,512,"%s %s\r\n",cmd,args);
        log_debug("IRC-OUT1",buff);
        mio_write(s->m,NULL,buff,strsz);
        return;
    }

    argsz = strlen(data);
    flag = 0;
    for(i = 1; i < argsz; i++)
    {
        if(data[i] == '\n')
        {
            if(data[i-1] == '\r')
                data[i-1] = '\0';
            data[i] = '\0';
            flag = 1;
            strsz = snprintf(buff,512,"%s %s:%s\r\n",cmd,args,data);
        }else if(i == 200){
            strsz = snprintf(buff,512,"%s %s:%.200s\r\n",cmd,args,data);
            flag = 1;
        }


        if(flag)
        {
            log_debug("IRC-OUT2",buff);
            mio_write(s->m,NULL,buff,strsz);
            flag = 0;
            data = data + i + 1;
            i = -1; /* get incremented by for */
            argsz = strlen(data);
        }
    }

    if(strlen(data) > 0)
    {
        strsz = snprintf(buff,512,"%s %s:%s\r\n",cmd,args,data);
        log_debug("IRC-OUT3",buff);
        mio_write(s->m,NULL,buff,strsz);
    }
}

/*
add a single char to a string
*/
char *addchar(pool p,char *in,char new)
/*
returns:
  the new string, with the char added
input:
  pool p:       a valid pool to use to add the char
  char *in:     a string to add the new char to
  char new:     a single character to add to the string
*/
{
    char *ret=pmalloc(p,2);
    spool sp=spool_new(p);
    sprintf(ret,"%c\0",new);
    if(in==NULL)
        spooler(sp,ret,sp);
    else
        spooler(sp,in,ret,sp);
    return spool_print(sp);
}

/*
converts a normal string to UTF8 format
*/
char *str_to_UTF8(pool p,unsigned char *in)
/*
returns:
  the new string, encoded in UTF-8
input:
  pool p:   a pool to define the life of the return value
  unsigned char *in: the str to convert
*/
{
    char *result=NULL;
    int n;
    if(in==NULL||p==NULL) return NULL;
    for(n=0;n<strlen(in);n++)
    {
        long c=(long)in[n];
        if(c<128)
        {
            result=addchar(p,result,(char)c);
        }
        else
        {
            result=addchar(p,result,(char)((c>>6)|192));
            result=addchar(p,result,(char)((c&63)|128));
        }
    }
    return result;
}

/*
convert a UTF-8 encoded string to a normal string
*/
unsigned char *UTF8_to_str(pool p,unsigned char *in)
/*
returns:
  a string not encoded in UTF-8
input:
  pool p: a pool to define the life of the return value
  unsigned char *in: the UTF-8 encoded string
*/
{
    int n;
    char *result=NULL;
    if(p==NULL||in==NULL) return NULL;
    n = 0;
    while(n<=strlen(in)-1)
    {
        long c=(long)in[n];
        if(c<0x80)
        {
            result=addchar(p,result,(char)c);
        }
        else
        {
            if((c&0xC0)==0xC0)
            {
                result=addchar(p,result,(char)(((c&0x03)<<6)|(((long)in[++n])&0x3F)));
            }
            else if((c&0xE0)==0xE0)
            {
                result=addchar(p,result,(char)(((c&0xF)<<4)|(((long)in[++n])&0x3F)|(((long)in[++n])&0x3F)));
            }
            else
                n++; /*skip that char*/
        }
        n++;
    }
        return result;
}

==> irc-transport-0.1.1-1.4.linux.tar.gz: irc-transport-0.1.1-1.40/src/util.c <==
/*
IRC-Transport util.c: general utility functions for IRC-transport 
Copyright (C) 1999-2000 Keith Minkler <tsbandit@jabber.org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

You can reach The author at tsbandit@jabber.org (email or jid)
*/

#include "irctrans.h"

/*
sets a mode for the user (ops and voice)
*/
void irc_set_mode(user u,int ops,int voice)
/*
returns;
 nothing
input:
 users u: the user to update
 int ops: the new ops vaule
 int voice: the new voice value
*/
{
    if(u==NULL) return;
    u->ops=ops;
    u->voice=voice;
}

/*
compares two nicknames, ignoreing ops and voice flags.
*/
int irc_nick_cmp(char *nick1,char *nick2)
/*
returns:
 -1 if either are NULL
 return value from strcasecmp for the two strings
input:
 char *nick1: duh
 char *nick2: duh
*/
{
    char *n1=nick1,*n2=nick2;
    if(n1==NULL||n2==NULL) return -1;
    if(n1[0]=='@'||n1[0]=='+') n1++;
    if(n2[0]=='@'||n2[0]=='+') n2++;
    return strcasecmp(n1,n2);
}

/*
strips out those nasty color codes
overwrites input with the new data
*/
void irc_strip_mirc_codes(char *input)
/*
returns:
 nothing
input:
 char *input: the string to convert, this gets overwritten iwth the result *MUST NOT BE READ ONLY* 
*/
{
    char *curr=input,*write=input;
    int a,w;
    long c;
    if(input==NULL) return;
    for(a=0,c=0,w=0;a<strlen(input);a++)
    {
        c=(long)curr[a];
        /*(  bold||underline||reverse||  plain)*/
        if(c==0x2||c==0x1F  ||c==0x16||c==0x0F)
        {
            /* skip this char */
            continue;
        }
        else if(c==0x3) /* one of these codes: # ## #,# #,## ##,# ##,## */
        { 
            c=curr[++a];
            if(isdigit(c))
            { /* # ## #,# #,## ##,# or ##,## */
                c=curr[++a];
                if(isdigit(c))
                { /* ## ##,# ##,## */
                    if(c==',')
                    { /* ##,# ##,## */
                        c=curr[a+2];
                        if(isdigit(c)) c=curr[++a];
                    }
                }
                else if(c==',')
                { /* #,# or #,## */    
                    c=curr[a+2];
                    if(isdigit(c)) c=curr[++a];
                }
            }
        }
        /* write the next char */
        write[w++]=(char)c;
    }
    write[w]='\0';
}

/* 
function to match '#Jabber' with 'jabber'
or '.jabber' with '&jabber'
NULL != NULL
*/
int irc_channel_cmp(const char *chan1,const char *chan2)
/*
returns:
 -1 if either are NULL
 the return value from strcasecmp
input:
 const char *char1,const char *char2: the values to compare
*/
{
    pool p;
    int ret=-1;
    char *c1,*c2;
    if(chan1==NULL||chan2==NULL) return ret;
    p=pool_new();
    c1=pstrdup(p,chan1);
    c2=pstrdup(p,chan2);
    /* normalize all channel names
        #foo => foo
        foo  => foo
        &foo => .foo
        .foo => .foo
    */
    if(c1[0]=='#') c1++;
    if(c1[0]=='&') c1[0]='.';
    if(c2[0]=='#') c2++;
    if(c2[0]=='&') c2[0]='.';
    if(c1!=NULL&&c2!=NULL)
        ret=strcasecmp(c1,c2);
    pool_free(p);
    return ret;
}

/*
extracts the channel name from a jid string
*/
char *irc_get_channel(pool p, char *input)
/*
returns:
 a pool'd copy o fthe channel using the supplied pool
 NULL if it could not find the channel
input:
 pool p: a pool to use to copy the data
 char *input: the jid string to extract from
*/
{
    char *curr,*next;
    char *ret=pstrdup(p,input);
    spool s;
    if(p==NULL||input==NULL) return NULL;
    curr=ret;
    next=strchr(curr,'%');
    if(next==NULL) next=strchr(curr,'@');
    if(next==NULL) return NULL;
    next[0]='\0';
    s=spool_new(p);
    if(curr[0]=='.') curr[0]='&';
    if(curr[0]!='#'&&curr[0]!='&') 
        spooler(s,"#",curr,s);
    else
        spooler(s,curr,s);

    ret = spool_print(s);
    make_lowercase(ret);
    return ret;
}

/*
extracts the server name from a jid string
*/
char *irc_get_server(pool p, jid j, char *input)
/*
returns:
 a pool'd copy of the server name using the supplied pool
 NULL if it could not find the server name
input:
 pool p: a pool to use to copy the data
 char *input: the jid string to extract from
*/
{
    jid temp;
    jid temp2;
    xmlnode x;
    char *servername;
    irctrans_data ircdata;

    char *curr,*next;
    char *ret=pstrdup(p,input);
    if(p==NULL||input==NULL) return NULL;
    curr=ret;
    next=strchr(curr,'%');
    if(next!=NULL) 
    {
        curr=next+1;
        ret=curr;
        next=strchr(curr,'%');
        if(next==NULL) next=strchr(curr,'@');
        if(next!=NULL) 
        {
            next[0]='\0';
            return ret;
        }
    }

    /* get default server */
    ircdata = (irctrans_data)j->next;
    temp = jid_new(j->p, jid_full(j));
    jid_set(temp, NULL, JID_RESOURCE);
    temp2 = jid_new(j->p, spools(j->p, shahash(jid_full(temp)), "@", ircdata->i->id, j->p));
    x = xdb_get(ircdata->xc, temp2, "irc:default:server");
    x = xmlnode_get_firstchild(x);
    servername = xmlnode_get_data(x) ? xmlnode_get_data(x) : xmlnode_get_tag_data(ircdata->cfg, "default/server");
    ret = pstrdup(p, servername);
    xmlnode_free(x);
    return ret;
}

/*
extracts the port name from a jid string
*/
char *irc_get_port(pool p,char *input)
/*
returns:
 a pool'd copy of the port string using the supplied pool
 NULL if it could not find the port
input:
 pool p: a pool to use to copy the data
 char *input: the jid string to extract from
*/
{
    char *curr,*next;
    char *ret=pstrdup(p,input);
    if(p==NULL||input==NULL) return NULL;
    curr=ret;
    next=strchr(curr,'%');
    if(next==NULL) 
        return pstrdup(p,xmlnode_get_tag_data(ircdata->cfg, "default/port"));    
    next=strchr(curr,'%');
    if(next==NULL)
        return pstrdup(p,xmlnode_get_tag_data(ircdata->cfg, "default/port"));    
    curr=next+1;
    next=strchr(curr,'@');
    if(next==NULL);
        return pstrdup(p,xmlnode_get_tag_data(ircdata->cfg, "default/port"));    
    ret=curr;
    next[0]='\0';
    return ret;
}

/* send an outgoing line to the connection */
void svc_irc_line(irc_session s, char *cmd, char *data, int argc, ...)
{
    static char buff[512], args[512];
    char *str;
    int strsz, i, argsz, flag;
    va_list ap;

    va_start(ap, argc);
    argsz = 0;
    args[argsz] = '\0';
    for(i = 0; i < argc; i++)
    {
        str = va_arg(ap,char *);
        if(str == NULL) continue;
        if(strlen(str) + argsz >= 500) continue;
        strcat(args,str);
        argsz += strlen(str);
        strcat(args," ");
        argsz++;
    }
    va_end(ap);

    if(data == NULL)
    {
        strsz = snprintf(buff,512,"%s %s\r\n",cmd,args);
        log_debug("IRC-OUT1",buff);
        mio_write(s->m,NULL,buff,strsz);
        return;
    }

    argsz = strlen(data);
    flag = 0;
    for(i = 1; i < argsz; i++)
    {
        if(data[i] == '\n')
        {
            if(data[i-1] == '\r')
                data[i-1] = '\0';
            data[i] = '\0';
            flag = 1;
            strsz = snprintf(buff,512,"%s %s:%s\r\n",cmd,args,data);
        }else if(i == 200){
            strsz = snprintf(buff,512,"%s %s:%.200s\r\n",cmd,args,data);
            flag = 1;
        }


        if(flag)
        {
            log_debug("IRC-OUT2",buff);
            mio_write(s->m,NULL,buff,strsz);
            flag = 0;
            data = data + i + 1;
            i = -1; /* get incremented by for */
            argsz = strlen(data);
        }
    }

    if(strlen(data) > 0)
    {
        strsz = snprintf(buff,512,"%s %s:%s\r\n",cmd,args,data);
        log_debug("IRC-OUT3",buff);
        mio_write(s->m,NULL,buff,strsz);
    }
}

/*
add a single char to a string
*/
char *addchar(pool p,char *in,char new)
/*
returns:
  the new string, with the char added
input:
  pool p:       a valid pool to use to add the char
  char *in:     a string to add the new char to
  char new:     a single character to add to the string
*/
{
    char *ret=pmalloc(p,2);
    spool sp=spool_new(p);
    sprintf(ret,"%c\0",new);
    if(in==NULL)
        spooler(sp,ret,sp);
    else
        spooler(sp,in,ret,sp);
    return spool_print(sp);
}

/*
converts a normal string to UTF8 format
*/
char *str_to_UTF8(pool p,unsigned char *in)
/*
returns:
  the new string, encoded in UTF-8
input:
  pool p:   a pool to define the life of the return value
  unsigned char *in: the str to convert
*/
{
    char *result=NULL;
    int n;
    if(in==NULL||p==NULL) return NULL;
    for(n=0;n<strlen(in);n++)
    {
        long c=(long)in[n];
        if(c<128)
        {
            result=addchar(p,result,(char)c);
        }
        else
        {
            result=addchar(p,result,(char)((c>>6)|192));
            result=addchar(p,result,(char)((c&63)|128));
        }
    }
    return result;
}

/*
convert a UTF-8 encoded string to a normal string
*/
unsigned char *UTF8_to_str(pool p,unsigned char *in)
/*
returns:
  a string not encoded in UTF-8
input:
  pool p: a pool to define the life of the return value
  unsigned char *in: the UTF-8 encoded string
*/
{
    int n;
    char *result=NULL;
    if(p==NULL||in==NULL) return NULL;
    n = 0;
    while(n<=strlen(in)-1)
    {
        long c=(long)in[n];
        if(c<0x80)
        {
            result=addchar(p,result,(char)c);
        }
        else
        {
            if((c&0xC0)==0xC0)
            {
                result=addchar(p,result,(char)(((c&0x03)<<6)|(((long)in[++n])&0x3F)));
            }
            else if((c&0xE0)==0xE0)
            {
                result=addchar(p,result,(char)(((c&0xF)<<4)|(((long)in[++n])&0x3F)|(((long)in[++n])&0x3F)));
            }
            else
                n++; /*skip that char*/
        }
        n++;
    }
        return result;
}

char *make_lowercase(char *input)
{
    if(input == NULL)
        return;

    if(input[0] == '\0')
        return;

    if(isalpha(*input))
        input[0] = tolower(*input);

    make_lowercase(input + 1);
    return input;
}
